package de.unikoblenz.fgbks.core.dmn.verification.metrics;

import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.VerificationType;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import javax.json.bind.annotation.JsonbProperty;

/**
 * A metric instance give statistics about the execution of one verifier {@link VerificationType}).
 */
public class Metric implements Serializable {

  public static final long NANO_SECONDS_PER_SECOND = 1_000_000L;
  VerificationType type;
  private List<Long> executionTimes;

  /**
   * Create a new metric instance
   */
  public Metric(VerificationType type) {
    this.type = type;
    this.executionTimes = new ArrayList<>();
  }

  /**
   * Get the number of total executions.
   *
   * @return the number of executions
   */
  @JsonbProperty("amountOfExecutions")
  public int getAmountOfExecutions() {
    return executionTimes.size();
  }

  /**
   * Get the average execution time in ns.
   *
   * @return the average execution time
   */
  @JsonbProperty("averageExecutionTime")
  public double getAverageExecutionTime() {
    return executionTimes.stream().mapToLong(Long::longValue).average().orElseGet(() -> 0);
  }

  /**
   * Get the average execution time in ns.
   *
   * @return the average execution time
   */
  @JsonbProperty("averageExecutionTimeInMs")
  public double getAverageExecutionTimeInMs() {
    return executionTimes.stream().mapToLong(Long::longValue).average().orElseGet(() -> 0)
        / NANO_SECONDS_PER_SECOND;
  }

  /** Get the total execution time in ns. */
  @JsonbProperty("totalExecutionTime")
  public long getTotalExecutionTime() {
    return executionTimes.stream().mapToLong(Long::longValue).sum();
  }

  /**
   * Get the {@link VerificationType}.
   *
   * @return the {@link VerificationType}
   */
  @JsonbProperty("type")
  public VerificationType getType() {
    return type;
  }

  /**
   * Get the total execution time in ms.
   *
   * @return the execution time in ms
   */
  @JsonbProperty("totalExecutionTimeInMs")
  public long getTotalExecutionTimeInMs() {
    return executionTimes.stream().mapToLong(Long::longValue).sum() / NANO_SECONDS_PER_SECOND;
  }

  /**
   * Add a new execution time.
   *
   * @param timeInMs the time in ns
   */
  void addExecutionTime(long timeInMs) {
    executionTimes.add(timeInMs);
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }
    Metric metric = (Metric) o;
    return type.equals(metric.type);
  }

  @Override
  public int hashCode() {
    return Objects.hash(type);
  }
}
